/* 
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: ...
*/


#include "animation.h"
#include "debug.h"
#include "list.h"


Anim::Anim(void)
{
  // create a new List
  this->keyFrameList = new tList<AnimKeyFrame>();
  
  // initialize a beginning KeyFrame, that will be deleted afterwards
  this->bHasKeys = false;
  AnimKeyFrame* tmpKeyFrame = new AnimKeyFrame;
  tmpKeyFrame->value = 0.0;
  tmpKeyFrame->duration = 1.0;
  keyFrameList->add(tmpKeyFrame);

  // setting default values
  this->localTime = 0.0;
  this->bRunning = true;
  this->animFunc = &Anim::linear;
  this->currentKeyFrame = tmpKeyFrame;
  this->nextKeyFrame = tmpKeyFrame;
}


Anim::~Anim(void)
{
  // delete all the KeyFrames
  tIterator<AnimKeyFrame>* itKF = keyFrameList->getIterator();
  AnimKeyFrame*  enumKF = itKF->nextElement();
  while (enumKF)
    {
      delete enumKF;
      enumKF = itKF->nextElement();
    }
  delete itKF;
  delete this->keyFrameList;
}

tList<Anim>* Anim::animatorList = NULL;


void Anim::addKeyFrame(float value, float duration, ANIM_FUNCTION animFunc)
{
  // some small check
  if (duration <= 0.0)
    duration = 1.0;
  

  AnimKeyFrame* tmpKeyFrame;
    
  if (bHasKeys)
    {
      tmpKeyFrame = new AnimKeyFrame;
      if (this->currentKeyFrame == this->nextKeyFrame)
	this->nextKeyFrame = tmpKeyFrame;
      this->keyFrameList->add(tmpKeyFrame);

    }
  else
    {
      tmpKeyFrame = this->keyFrameList->firstElement();
      bHasKeys = true;
    }
  tmpKeyFrame->value = value;
  tmpKeyFrame->duration = duration;
  tmpKeyFrame->animFunc = animFunc;
  
}

void Anim::setInfinity(ANIM_INFINITY postInfinity)
{
  this->postInfinity = postInfinity;
}

void Anim::setAnimFunc(ANIM_FUNCTION animFunc)
{
  switch (animFunc)
    {
    default:
    case ANIM_CONSTANT:
      this->animFunc = &Anim::constant;
      break;
    case ANIM_LINEAR:
      this->animFunc = &Anim::linear;
      break;
    case ANIM_RANDOM:
      this->animFunc = &Anim::random;
      break;
    case ANIM_SINE:
      this->animFunc = &Anim::sine;
      break;
    }
}


// animation functions
float Anim::random(float timePassed) const
{
  return (float)rand()/(float)RAND_MAX;
}

float Anim::constant(float timePassed) const
{
  return this->currentKeyFrame->value;
}

float Anim::linear(float timePassed) const 
{
  return this->nextKeyFrame->value - (this->nextKeyFrame->value - this->currentKeyFrame->value) 
    * (timePassed / this->currentKeyFrame->duration);
  //  PRINTF(0)("value is %f, %p %p\n", val, this->currentKeyFrame, this->nextKeyFrame);
  //  return val;
}

float Anim::sine(float timePassed) const
{
  
}
